package Plack::App::FakeApache::Server;
$Plack::App::FakeApache::Server::VERSION = '0.1';
use Moo;
use namespace::autoclean;
use Plack::App::FakeApache::Log;

has log => (
    is      => 'rw',
    default => sub { Plack::App::FakeApache::Log->new() },
    handles => [ qw(log_error log_serror warn) ],
);

1;
