use Test::More;

require_ok( 'Plack::App::FakeApache' );
require_ok( 'Plack::App::FakeApache::Request' );
require_ok( 'Plack::App::FakeApache::Connection' );
require_ok( 'Plack::App::FakeApache::Log' );
done_testing;

